// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//  Giuseppe Vacanti (cosine science & computing bv)
//  December  7, 2002
// 
//   $Id: sasparam.cc,v 1.2 2003/02/07 15:53:29 gvacanti Exp $
//
// Command line interface to the param library.
//

#include <param.h>
#include <errstr.h>
#include <XmlInfoReader.h>
#include <SasFileLocator.h>

#include <boost/lexical_cast.hpp>

#include <string>
#include <iostream>
#include <stdexcept>

// sasparam client -c param -> parameterCount
// sasparam client -i param [count] -> intParameter
// sasparam client -r param [count] -> realParameter
// sasparam client -b param [count] -> boolParameter
// sasparam client -s param [count] -> stringParameter
// sasparam client -h -> displays help

namespace {
    void internalError(const std::string & s){
	errHandler.name("sasparam");
	Sas::error("InternalError", s + std::string(" Report this."));
    }
    void usageError(const std::string & s){
	errHandler.name("sasparam");
	Sas::error("UsageError", s);
    }

};


int main(int argc, char * argv[]){

    if(argc < 3)
	internalError("Wrong calling sequence.");
	
    std::string client(argv[1]);
//    openParameters(client);
    std::string lookfor("config/" + client + ".par");
    std::string parfile = fileLocator.locate(lookfor);
    XmlInfoReader reader(parfile,false);
    ParameterInfo & info = *reader.getParamInfo();

    std::string what(argv[2]);

    // Just the help and exit
    if(argc == 3)
	if(what == "-h"){
	    info.writeHelpToStream(std::cout);
	    return 0;
	} else
	    internalError("Wrong calling sequence.");
    
    std::string parameter(argv[3]);
    int count = -1; // I rely here on the knowledge that count = -1
		    // means don't count (see param.h).
    // Decide if the 5th argument is an integer, in which case it is
    // refers to the parameter count.
    bool hascount = false;
    if(argc >= 5){
	using boost::lexical_cast;
	try {
	    count = lexical_cast<int>(std::string(argv[4]));
	    hascount = true;
	}
	catch(boost::bad_lexical_cast & e){
//            internalError(e.what());
        }
    }
    if(hascount){
	argc -= 5;
	argv += 5;
    } else {
	argc -= 4;
	argv += 4;
    }

//     for(int k = 0; k < argc; ++k)
// 	std::cout << "DEBUG: " << k << " " << argv[k] << std::endl;

    info.parseCommandLine(argc, argv);
    info.checkMandatoryParameters();

//     std::cout << info << std::endl;
//     std::cout << info.asCommandLine() << std::endl;

    Parameter * p = info[parameter];

    // gcc-2.95.3 does not have at()

    try {
	if(what == "-c")
	    std::cout << p->size();
	else if(what == "-i"){
	    std::cout << (count >= 0 ? p->intListValue()[count]
			  : p->intValue());
// 	    std::cout << (count >= 0 ? p->intListValue().at(count)
// 			  : p->intValue());
	} else if(what == "-r") {
// 	    std::cout << (count >= 0 ? p->realListValue().at(count)
// 			  : p->realValue());
	    std::cout << (count >= 0 ? p->realListValue()[count]
			  : p->realValue());
	} else if(what == "-b") {
// 	    bool value = (count >= 0 ? p->boolListValue().at(count)
// 			  : p->boolValue());
	    bool value = (count >= 0 ? p->boolListValue()[count]
			  : p->boolValue());
	    std::cout << (value ? 1 : 0);
	} else if(what == "-s") {
// 	    std::cout << (count >= 0 ? p->stringListValue().at(count)
// 			  : p->stringValue());
	    std::cout << (count >= 0 ? p->stringListValue()[count]
			  : p->stringValue());
	} else
	    internalError("Cannot handle parameter type.");
    }
    catch(std::out_of_range & e){
	usageError("Caught an out of range exception: " +
		      std::string(e.what()) +
		      ". Probably you tried to access a parameter list with an index out of range.");
    }
    catch(...){
	internalError("Unexpected exception.");
    }
    std::cout << std::endl;
}
